@@ -18,12 +18,16 @@ import datetime
 import logging
 import re
 
+from oslo_serialization import jsonutils
 from oslo_utils import strutils
 
 from blazarclient import command
 from blazarclient import exception
 
 
+# All valid reservation parameters must be added to CREATE_RESERVATION_KEYS to
+# make them parsable. Note that setting the default value to None ensures that
+# the parameter is not included in the POST request if absent.
 CREATE_RESERVATION_KEYS = {
     "physical:host": {
         "min": "",
@@ -33,6 +37,12 @@ CREATE_RESERVATION_KEYS = {
         "before_end": None,
         "resource_type": 'physical:host'
     },
+    "virtual:floatingip": {
+        "amount": 1,
+        "network_id": None,
+        "required_floatingips": [],
+        "resource_type": 'virtual:floatingip'
+    },
     "virtual:instance": {
         "vcpus": "",
         "memory_mb": "",
@@ -48,6 +58,16 @@ CREATE_RESERVATION_KEYS = {
 }
 
 
+def _utc_now():
+    """Wrap datetime.datetime.utcnow so it can be mocked in unit tests.
+
+    This is required because some of the tests require understanding the
+    'current time'; simply mocking utcnow() is made very difficult by
+    the many different ways the datetime package is used in this module.
+    """
+    return datetime.datetime.utcnow()
+
+
 class ListLeases(command.ListCommand):
     """Print a list of leases."""
     resource = 'lease'
@@ -77,7 +97,7 @@ class CreateLease(command.CreateCommand):
     json_indent = 4
     log = logging.getLogger(__name__ + '.CreateLease')
     default_start = 'now'
-    default_end = datetime.datetime.utcnow() + datetime.timedelta(days=1)
+    default_end = _utc_now() + datetime.timedelta(days=1)
 
     def get_parser(self, prog_name):
         parser = super(CreateLease, self).get_parser(prog_name)
@@ -163,6 +183,8 @@ class CreateLease(command.CreateCommand):
                 else:
                     if strutils.is_int_like(v):
                         request_params[k] = int(v)
+                    elif isinstance(defaults[k], list):
+                        request_params[k] = jsonutils.loads(v)
                     else:
                         request_params[k] = v
 
@@ -177,18 +199,25 @@ class CreateLease(command.CreateCommand):
         if parsed_args.name:
             params['name'] = parsed_args.name
         if not isinstance(parsed_args.start, datetime.datetime):
-            try:
-                parsed_args.start = datetime.datetime.strptime(
-                    parsed_args.start, '%Y-%m-%d %H:%M')
-            except ValueError:
-                raise exception.IncorrectLease
+            if parsed_args.start != 'now':
+                try:
+                    parsed_args.start = datetime.datetime.strptime(
+                        parsed_args.start, '%Y-%m-%d %H:%M')
+                except ValueError:
+                    raise exception.IncorrectLease
         if not isinstance(parsed_args.end, datetime.datetime):
             try:
                 parsed_args.end = datetime.datetime.strptime(
                     parsed_args.end, '%Y-%m-%d %H:%M')
             except ValueError:
                 raise exception.IncorrectLease
-        if parsed_args.start > parsed_args.end:
+
+        if parsed_args.start == 'now':
+            start = _utc_now()
+        else:
+            start = parsed_args.start
+
+        if start > parsed_args.end:
             raise exception.IncorrectLease
 
         if parsed_args.before_end:
@@ -197,14 +226,17 @@ class CreateLease(command.CreateCommand):
                     parsed_args.before_end, '%Y-%m-%d %H:%M')
             except ValueError:
                 raise exception.IncorrectLease
-            if (parsed_args.before_end < parsed_args.start
-                    or parsed_args.end < parsed_args.before_end):
+            if (parsed_args.before_end < start or
+                    parsed_args.end < parsed_args.before_end):
                 raise exception.IncorrectLease
             params['before_end'] = datetime.datetime.strftime(
                 parsed_args.before_end, '%Y-%m-%d %H:%M')
 
-        params['start'] = datetime.datetime.strftime(parsed_args.start,
-                                                     '%Y-%m-%d %H:%M')
+        if parsed_args.start == 'now':
+            params['start'] = parsed_args.start
+        else:
+            params['start'] = datetime.datetime.strftime(parsed_args.start,
+                                                         '%Y-%m-%d %H:%M')
         params['end'] = datetime.datetime.strftime(parsed_args.end,
                                                    '%Y-%m-%d %H:%M')
 
@@ -265,6 +297,8 @@ class CreateLease(command.CreateCommand):
                 defaults = CREATE_RESERVATION_KEYS['physical:host']
             elif "virtual:instance" in res_str:
                 defaults = CREATE_RESERVATION_KEYS['virtual:instance']
+            elif "virtual:floatingip" in res_str:
+                defaults = CREATE_RESERVATION_KEYS['virtual:floatingip']
             else:
                 defaults = CREATE_RESERVATION_KEYS['others']
 
@@ -388,14 +422,17 @@ class UpdateLease(command.UpdateCommand):
         if parsed_args.start_date:
             params['start_date'] = parsed_args.start_date
         if parsed_args.reservation:
-            keys = [
+            keys = set([
                 # General keys
                 'id',
                 # Keys for host reservation
                 'min', 'max', 'hypervisor_properties', 'resource_properties',
                 # Keys for instance reservation
-                'vcpus', 'memory_mb', 'disk_gb', 'amount', 'affinity'
-            ]
+                'vcpus', 'memory_mb', 'disk_gb', 'amount', 'affinity',
+                # Keys for floating IP reservation
+                'amount', 'network_id', 'required_floatingips',
+            ])
+            list_keys = ['required_floatingips']
             params['reservations'] = []
             reservations = []
             for res_str in parsed_args.reservation:
@@ -409,7 +446,9 @@ class UpdateLease(command.UpdateCommand):
                     match = prog.search(params)
                     if match:
                         k, v = match.group(2, 3)
-                        if strutils.is_int_like(v):
+                        if k in list_keys:
+                            v = jsonutils.loads(v)
+                        elif strutils.is_int_like(v):
                             v = int(v)
                         res_info[k] = v
                         if match.group(1) is not None:
